@@ -6,7 +6,8 @@ a = symbol('a')
 f0 = symbol('f0')
 w0 = 2 * pi * f0
 
-sigs = [cos(w0 * n * dt), sin(w0 * n * dt), exp(j * w0 * n * dt),
+sigs = [nexpr('x(n)'), nexpr('x(n - m)'),
+        cos(w0 * n * dt), sin(w0 * n * dt), exp(j * w0 * n * dt),
         nexpr(1), delta(n), delta(n - p), a**n, a**-n, n * a**n, n * a**-n,
         H(n), exp(-n * dt) * H(n)]
 
