@@ -55,6 +55,7 @@ from .smatrix import *
 from .tmatrix import *
 from .vector import *
 from .statespace import *
+from .dtstatespace import *
 from .nodalanalysis import *
 from .loopanalysis import *
 from .laplace import *
@@ -75,6 +76,8 @@ from .diffeq import *
 from .dft import *
 from .inverse_dft import *
 from .state import state
+# Do not import units.u since this will conflict with unit step
+from .units import volts, amperes, ohms, siemens, watts
 
 def show_version():
     """Show versions of Lcapy, SymPy, NumPy, MatplotLib, SciPy, and Python."""
