@@ -32,16 +32,20 @@ functions = ('heaviside', 'diracdelta', 'conjugate', 'sqrt', 'exp',
              'log', 'log10', 'sin', 'cos', 'tan', 'cot' 'asin',
              'acos', 'atan', 'atan2', 'acot', 'sinh', 'cosh', 'tanh', 'asinh',
              'acosh', 'atanh', 'gcd', 'abs', 'unitimpulse', 'arg', 'sign',
-             'rect', 'sinc', 'sincn', 'sincu', 'trap', 'tri', 'dtrect', 'dtsign')
+             'rect', 'sinc', 'sincn', 'sincu', 'trap', 'tri',
+             'dtrect', 'dtsign', 'psinc')
 
 subscripts = ('in', 'out', 'ref', 'rms', 'load', 'source', 'avg',
               'mean', 'peak', 'pk', 'pk-pk', 'pp', 'min', 'max', 'src', 'bat',
               'cc', 'ee', 'dd', 'ss', 'ih', 'il', 'oh', 'ol',
               'typ', 'pkg', 'comp', 'step')
 
-words = ('alpha', 'beta', 'gamma', 'delta', 'eta', 'zeta', 'theta',
-         'iota', 'kappa', 'mu', 'nu', 'omicron', 'pi', 'rho', 'sigma', 'tau',
-         'upsilon', 'omega')
+greek_letter_names = ('alpha', 'beta', 'gamma', 'delta', 'eta', 'zeta',
+                      'theta', 'iota', 'kappa', 'lambda', 'mu', 'nu',
+                      'omicron', 'pi', 'rho', 'sigma', 'tau',
+                      'upsilon', 'chi', 'psi', 'omega')
+
+words = greek_letter_names
 
 # Can be 'GE', 'LU', 'ADJ', 'LDL', 'CH', 'DM-GE', 'DM-LU', 'DM-charpoly'
 # Note, the DM methods require the git version of sympy otherwise
