@@ -15,7 +15,8 @@ __all__ = ('conjugate', 'sqrt', 'exp', 'log', 'log10', 'sin', 'cos',
            'us', 'unitimpulse', 'unitstep', 'UnitImpulse', 'UnitStep',
            'rect', 'sinc', 'sincn', 'sincu', 'psinc', 'tri', 'trap',
            'Sum', 'dtrect', 'dtsign', 'Piecewise', 'Eq', 'Ne',
-           'Lt', 'Le', 'Gt', 'Ge')
+           'Lt', 'Le', 'Gt', 'Ge', 'Derivative', 'Integral', 'Max', 'Min',
+           're', 'im')
 
 from .extrafunctions import UnitImpulse as UnitImpulse1
 from .extrafunctions import UnitStep as UnitStep1
@@ -28,10 +29,13 @@ from .extrafunctions import dtsign as dtsign1
 from .extrafunctions import tri as tri1
 from .extrafunctions import trap as trap1
 
-
 class Function(object):
 
     def __init__(self, arg):
+
+        if isinstance(arg, str):
+            # Handled AppliedUndef
+            arg = sym.Function(arg)
         self.expr = arg
     
     @property
@@ -40,7 +44,7 @@ class Function(object):
         
         return self.expr
 
-    def __call__(self, *args):
+    def __call__(self, *args, **kwargs):
 
         func = self.expr
             
@@ -48,6 +52,13 @@ class Function(object):
 
         if func == sym.Piecewise:
             cls = e_args[0][0].__class__
+        elif func in (sym.Integral, sym.integrate):
+            # Integral(const, (tau, 0, t)) -> use t class
+            # Integral(integrand, (tau, a, b)) -> use integrand class
+            if e_args[0].is_constant:
+                cls = e_args[1][2].__class__                
+            else:
+                cls = e_args[0].__class__            
         else:
             cls = e_args[0].__class__
             
@@ -59,19 +70,15 @@ class Function(object):
         try:
             if cls.is_discrete_time_domain or cls.is_discrete_fourier_domain:
 
-                mapping = {sym.Heaviside: UnitStep1,
-                           sym.DiracDelta: UnitImpulse1,
-                           rect1: dtrect,
-                           sym.sign: dtsign}
-                
-                for old, new in mapping.items():
+                # TODO: fix for Heaviside change
+                for old, new in function_mapping.items():
                     if func is old:
                         func = new
                         break
         except:
             pass
                 
-        result = func(*delcapify(e_args))
+        result = func(*delcapify(e_args), **kwargs)
 
         result = cls(result)        
             
@@ -178,6 +185,10 @@ delta = DiracDelta = Function(sym.DiracDelta)
 
 Piecewise = Function(sym.Piecewise)
 
+Derivative = Function(sym.Derivative)
+
+Integral = Function(sym.Integral)
+
 Eq = Function(sym.Eq)
 
 Ne = Function(sym.Ne)
@@ -200,6 +211,14 @@ MatMul = Function(sym.MatMul)
 
 Sum = Function(sym.Sum)
 
+Min = Function(sym.Min)
+
+Max = Function(sym.Max)
+
+re = Function(sym.re)
+
+im = Function(sym.im)
+
 ui = unitimpulse = UnitImpulse = Function(UnitImpulse1)
 
 us = unitstep = UnitStep = Function(UnitStep1)
@@ -221,6 +240,10 @@ psinc = PsincFunction(psinc1)
 tri = Function(tri1)
 
 trap = Function(trap1)
- 
+
+function_mapping = {sym.Heaviside: UnitStep1,
+                    sym.DiracDelta: UnitImpulse1,
+                    rect1: dtrect,
+                    sym.sign: dtsign}
 
 from .expr import Expr, expr, delcapify
