@@ -6,9 +6,10 @@ class ImpedanceMixin(Quantity, ImmittanceMixin):
 
     quantity = 'impedance'
     quantity_label = 'Impedance'
-    units = 'ohm'
+    quantity_units = 'ohm'
     is_impedance = True
-    is_immitance = True
+    is_immittance = True
+    is_ratio = True
 
     # Immittances derived from a realisable circuit will be causal but
     # non-causal immittances can also be constructed.  So this might
