@@ -7,17 +7,22 @@ Copyright 2021 Michael Hayes, UCECE
 
 from .transformer import UnilateralInverseTransformer
 from .ratfun import Ratfun
-from .sym import sympify, simplify, AppliedUndef
+from .sym import simplify, AppliedUndef
 from .utils import factor_const, scale_shift, as_sum_terms
+from .matrix import matrix_inverse
+from warnings import warn
 import sympy as sym
 
+Zero = sym.S.Zero
+One = sym.S.One
+
 __all__ = ('ILT', 'inverse_laplace_transform')
 
 
 class InverseLaplaceTransformer(UnilateralInverseTransformer):
 
     name = 'inverse Laplace transform'
-    
+
     def noevaluate(self, expr, s, t):
 
         # Construct Bromwich integral.
@@ -31,26 +36,24 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
     def key(self, expr, s, t, **kwargs):
         return (expr, s, t,
                 kwargs.get('causal', False),
-                kwargs.get('damped_sin', True),            
+                kwargs.get('damped_sin', True),
                 kwargs.get('damping', None))
 
-    def func(self, expr, t, s):
+    def func(self, expr, s, t):
 
         if not isinstance(expr, AppliedUndef):
             self.error('Expecting function')
 
-        scale, shift = scale_shift(expr.args[0], t)    
-
-        ssym = sympify(str(s))
+        scale, shift = scale_shift(expr.args[0], s)
 
         # Convert V(s) to v(t), etc.
         name = expr.func.__name__
-        func = name[0].lower() + name[1:] + '(%s)' % s
+        undef = sym.Function(name[0].lower() + name[1:])(t)
 
-        result = sympify(func).subs(ssym, s / scale) / abs(scale)
+        result = undef.subs(t, t / scale) / abs(scale)
 
         if shift != 0:
-            result = result * sym.exp(s * shift / scale)    
+            result = result * sym.exp(t * shift / scale)
         return result
 
     def do_damped_sin(self, expr, s, t):
@@ -59,7 +62,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         K = ncoeffs[0] / dcoeffs[0]
 
         ncoeffs = [(c / ncoeffs[0]) for c in ncoeffs]
-        dcoeffs = [(c / dcoeffs[0]) for c in dcoeffs]        
+        dcoeffs = [(c / dcoeffs[0]) for c in dcoeffs]
 
         if len(ncoeffs) > 3 or len(dcoeffs) > 3:
             self.error('Not a second-order response')
@@ -68,7 +71,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         zeta = dcoeffs[1] / (2 * omega0)
 
         if zeta.is_constant() and zeta > 1:
-            print('Warning: expression is overdamped')
+            warn('Expression is overdamped')
 
         sigma1 = (zeta * omega0).simplify()
         omega1 = (omega0 * sym.sqrt(1 - zeta**2)).simplify()
@@ -83,7 +86,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         #h = K * sym.exp(-sigma1 * t) * sym.sinh(omega0 * mu * t)
 
         if len(ncoeffs) == 1:
-            return sym.S.Zero, h
+            return Zero, h
 
         C = sym.cos(omega1 * t)
         kCd = omega1
@@ -91,7 +94,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         hd = K * E * (kCd * C + kSd * S)
 
         if len(ncoeffs) == 2:
-            return sym.S.Zero, K * E * (kCd * C + (ncoeffs[1] + kSd) * S)
+            return Zero, K * E * (kCd * C + (ncoeffs[1] + kSd) * S)
 
         kCdd = -2 * omega1 * sigma1
         kSdd = sigma1**2 - omega1**2
@@ -102,6 +105,9 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
 
     def ratfun(self, expr, s, t, **kwargs):
 
+        if kwargs.pop('pdb', False):
+            import pdb; pdb.set_trace()
+
         sexpr = Ratfun(expr, s)
 
         if kwargs.get('damped_sin', False):
@@ -110,71 +116,62 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
             #if False and sexpr.degree == 3 and Ratfun(expr * s).degree == 2:
             #    return self.do_damped_sin3(sexpr, s, t)
 
-        Q, M, D, delay, undef = sexpr.as_QMD()
+        self.debug('Finding QRPO representation')
 
-        cresult = sym.S.Zero
+        damping = kwargs.get('damping', None)
+        Q, R, P, O, delay, undef = sexpr.as_QRPO(damping)
+
+        if delay != 0:
+            # This will be caught and trigger expansion of the expression.
+            raise ValueError('Unhandled delay %s' % delay)
+
+        cresult = Zero
 
         if Q:
-            Qpoly = sym.Poly(Q, s)        
+            Qpoly = sym.Poly(Q, s)
             C = Qpoly.all_coeffs()
             for n, c in enumerate(C):
                 cresult += c * sym.diff(sym.DiracDelta(t), t, len(C) - n - 1)
 
-        expr = M / D
-        for factor in expr.as_ordered_factors():
-            if factor == sym.oo:
-                return factor
+        if R == []:
+            return cresult, 0
 
-        sexpr = Ratfun(expr, s)
-        poles = sexpr.poles(damping=kwargs.get('damping', None))
+        uresult = 0
+        for m, (p, n, A) in enumerate(zip(P, O, R)):
 
-        polesdict = {}
-        for pole in poles:
-            polesdict[pole.expr] = pole.n
-
-        uresult = sym.S.Zero
-
-        for pole in poles:
-
-            p = pole.expr
-
-            # Number of occurrences of the pole.
-            o = polesdict[p]        
-
-            if o == 0:
+            # This is zero for the conjugate pole.
+            if R[m] == 0:
                 continue
 
-            if o == 1:
-                pc = pole.conj
-                r = sexpr.residue(p, poles)
+            # Search and remove conjugate pair.
+            has_conjpair = False
+            if p.is_complex and kwargs.get('pairs', True):
+                pc = p.conjugate()
+                Ac = A.conjugate()
+                for m2, p2 in enumerate(P[m + 1:]):
+                    m2 += m + 1
+                    if n == O[m2] and p2 == pc and R[m2] == Ac:
+                        R[m2] = 0
+                        has_conjpair = True
+                        break
 
-                if pc != p and pc in polesdict:
-                    # Remove conjugate from poles and process pole with its
-                    # conjugate.  Unfortunately, for symbolic expressions
-                    # we cannot tell if a quadratic has two real poles,
-                    # a repeated real pole, or a complex conjugate pair of poles.
+            if has_conjpair:
+                # Combine conjugate pairs.
+                p = p.expand(complex=True)
+                A = A.expand(complex=True)
+                p_re = sym.re(p)
+                p_im = sym.im(p)
+                A_re = sym.re(A)
+                A_im = sym.im(A)
+                et = sym.exp(p_re * t)
+                result = 2 * A_re * et * sym.cos(p_im * t)
+                result -= 2 * A_im * et * sym.sin(p_im * t)
+            else:
+                result = A * sym.exp(p * t)
 
-                    polesdict[pc] -= 1
-
-                    p_re = sym.re(p)
-                    p_im = sym.im(p)
-                    r_re = sym.re(r)
-                    r_im = sym.im(r)
-                    et = sym.exp(p_re * t)
-                    uresult += 2 * r_re * et * sym.cos(p_im * t)
-                    uresult -= 2 * r_im * et * sym.sin(p_im * t)
-                else:
-                    uresult += r * sym.exp(p * t)
-                continue
-
-            # Handle repeated poles.
-            expr2 = expr * (s - p) ** o
-            expr2 = expr2.simplify()        
-            for n in range(1, o + 1):
-                m = o - n
-                r = sym.limit(
-                    sym.diff(expr2, s, m), s, p) / sym.factorial(m)
-                uresult += r * sym.exp(p * t) * t**(n - 1)
+            if n > 1:
+                result *= t ** (n - 1) / sym.factorial(n - 1)
+            uresult += result
 
         # cresult is a sum of Dirac deltas and its derivatives so is known
         # to be causal.
@@ -187,11 +184,11 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         # / s etc.
         if kwargs.get('causal', False):
             # Assume that all functions are causal in the expression.
-            t1 = sym.S.Zero
+            t1 = Zero
             t2 = t
         else:
             t1 = -sym.oo
-            t2 = sym.oo        
+            t2 = sym.oo
 
         const, expr = factor_const(expr, s)
 
@@ -210,7 +207,7 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
             # Try to expose more simple cases, e.g. (R + s * L) * V(s)
             terms = factors[0].expand().as_ordered_terms()
             if len(terms) >= 2:
-                result = sym.S.Zero
+                result = Zero
                 for term in terms:
                     result += self.product(factors[1] * term, s, t, **kwargs)
                 return result * const
@@ -224,23 +221,23 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
                 # Note, as_ordered_factors puts powers of s before the functions.
                 if factors[0] == s:
                     # Handle differentiation
-                    # Convert s * V(s) to d v(t) / dt                        
-                    result = self.func(factors[1], s, t)            
+                    # Convert s * V(s) to d v(t) / dt
+                    result = self.func(factors[1], s, t)
                     result = sym.Derivative(result, t)
                     continue
                 elif factors[0].is_Pow and factors[0].args[0] == s and factors[0].args[1] > 0:
                     # Handle higher order differentiation
                     # Convert s ** 2 * V(s) to d^2 v(t) / dt^2
-                    result = self.func(factors[1], s, t)            
+                    result = self.func(factors[1], s, t)
                     result = sym.Derivative(result, t, factors[0].args[1])
-                    continue                
+                    continue
                 elif factors[0].is_Pow and factors[0].args[0] == s and factors[0].args[1] == -1:
                     # Handle integration  1 / s * V(s)
                     tau = self.dummy_var(expr, 'tau', level=intnum, real=True)
                     intnum += 1
                     result = self.func(factors[1], s, tau)
                     result = sym.Integral(result, (tau, t1, t))
-                    continue                
+                    continue
             # Convert product to convolution
             tau = self.dummy_var(expr, 'tau', level=intnum, real=True)
             intnum += 1
@@ -275,8 +272,8 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
 
     def delay_factor(self, expr, var):
 
-        delay = sym.S.Zero    
-        rest = sym.S.One
+        delay = Zero
+        rest = One
 
         for f in expr.as_ordered_factors():
             b, e = f.as_base_exp()
@@ -294,13 +291,14 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
 
     def sympy(self, expr, s, t):
 
+        self.debug('Resorting to SymPy')
+
         # This barfs when needing to generate Dirac deltas
-        from sympy.integrals.transforms import laplace_transform
-        result = laplace_transform(expr, t, s)
+        from sympy.integrals.transforms import inverse_laplace_transform
+        result = inverse_laplace_transform(expr, s, t)
 
         if result.has(sym.InverseLaplaceTransform):
-            self.error('SymPy cannot do it')
-            
+            raise ValueError('SymPy cannot find inverse Laplace transform of %s' % expr)
         return result
 
     def term1(self, expr, s, t, **kwargs):
@@ -311,10 +309,10 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
             # Handle V(s), 3 * V(s) etc.  If causal is True it is assumed
             # that the unknown functions are causal.
             result = self.func(expr, s, t)
-            return result * const, sym.S.Zero
+            return result * const, Zero
 
         if expr.has(AppliedUndef):
-            return const * self.product(expr, s, t, **kwargs), sym.S.Zero
+            return const * self.product(expr, s, t, **kwargs), Zero
 
         try:
             # This is the common case.
@@ -323,36 +321,94 @@ class InverseLaplaceTransformer(UnilateralInverseTransformer):
         except:
             pass
 
-        try:
-            return sym.S.Zero, const * self.sympy(expr, s, t)
-        except:
-            pass
-
         if expr.is_Pow and expr.args[0] == s:
-            return sym.S.Zero, const * self.power(expr, s, t)
+            return Zero, const * self.power(expr, s, t)
 
-        # As last resort see if can convert to convolutions...
-        return sym.S.Zero, const * self.product(expr, s, t, **kwargs)
+        raise ValueError('Cannot determine inverse Laplace transform')
 
     def term(self, expr, s, t, **kwargs):
 
         expr, delay = self.delay_factor(expr, s)
 
-        cresult, uresult = self.term1(expr, s, t, **kwargs)
+        if delay == 0 and expr.has(sym.exp):
+            # Handle cases like 1 / (s**2 * exp(5 * s) + s * exp(5 * s))
+            expr1 = expr.simplify()
+            expr2, delay2 = self.delay_factor(expr1, s)
+            if not expr2.has(sym.exp):
+                # Simplify can make things worse, e.g., 1 - exp(-5 *s)
+                # becomes exp(-5 * s) * (exp(5 * s) - 1)
+                expr = expr2
+                delay = delay2
+
+        try:
+            cresult, uresult = self.term1(expr, s, t, **kwargs)
+        except:
+
+            # With deep=True, SymPy makes mess of (1 - exp(-s * T))
+            terms = expr.expand(deep=False).as_ordered_terms()
+
+            if len(terms) > 1:
+
+                try:
+                    # See if can convert to convolutions...
+                    return self.product(expr, s, t, **kwargs), Zero
+                except:
+                    pass
+
+                uresult = Zero
+                cresult = Zero
+
+                for term in terms:
+                    term = term.simplify()
+                    cterm, uterm = self.term(term, s, t, **kwargs)
+                    cresult += cterm
+                    uresult += uterm
+                return cresult, uresult
+
+            expr = expr.simplify()
+
+            try:
+                cresult, uresult = self.term1(expr, s, t, **kwargs)
+            except:
+                return Zero, self.sympy(expr, s, t)
 
         if delay != 0:
             cresult = cresult.subs(t, t - delay)
             uresult = uresult.subs(t, t - delay)
 
-        # TODO, should check for delay < 0.  If so the causal
-        # part is no longer causal.
+            # h(t) = g(t - T)
+            # If h(t) is known to be causal and T >= 0, then g(t)
+            # is also causal.  If T > 0, then causality is violated.
 
-        if kwargs.get('causal', False):
-            uresult = uresult * sym.Heaviside(t - delay)
+            # If h(t) is not known to be causal then we can only infer
+            # it for t >= 0 from its Laplace transform H(s).
+            # From the delay theorem, H(s) = G(s) * exp(-s * T).
+            # So given G(s) we can only infer g(t) for t >= 0.
+            # This implies that we can only infer h(t) for t >= T.
+            # This creates a can of worms since different terms of
+            # an expression can have different conditions, say
+            # exp(-3 * s) / s**2 + exp(-4 * s) / s.  So for now,
+            # assume causal expression...
+
+            if not kwargs.get('causal', False) and expr.has(AppliedUndef):
+                warn('Assuming causal expression')
+
+            if not delay.is_negative:
+                if not delay.is_positive:
+                    warn('Assuming %s is positive' % delay)
+                cresult += uresult * sym.Heaviside(t - delay)
+                uresult = Zero
+            else:
+                raise ValueError('Causality violated with time advance %s.' % delay)
+
+        else:
+            if kwargs.get('causal', False):
+                cresult += uresult * sym.Heaviside(t)
+                uresult = Zero
 
         return cresult, uresult
 
-    
+
 inverse_laplace_transformer = InverseLaplaceTransformer()
 
 
@@ -366,7 +422,7 @@ def inverse_laplace_transform(expr, s, t, evaluate=True, **kwargs):
     `causal` -- x(t) = 0 for n < 0.
     `ac` -- x(t) = A cos(a * n) + B * sin(b * n)
     """
-    
+
     return inverse_laplace_transformer.transform(expr, s, t,
                                                  evaluate=evaluate,
                                                  **kwargs)
