@@ -15,7 +15,7 @@ that it gives DiracDelta(t) for the inverse Laplace transform of 1.
 Another difference with this implementation is that it will transform
 undefined functions such as v(t) to V(s).
 
-These functions are for internal use by Lcapy.  
+These functions are for internal use by Lcapy.
 
 Copyright 2016--2021 Michael Hayes, UCECE
 
@@ -31,8 +31,8 @@ __all__ = ('LT', 'laplace_transform')
 
 class LaplaceTransformer(UnilateralForwardTransformer):
 
-    name = 'inverse Laplace transform'
-    
+    name = 'Laplace transform'
+
     def noevaluate(self, expr, t, s):
 
         t0 = sympify('t0')
@@ -80,7 +80,7 @@ class LaplaceTransformer(UnilateralForwardTransformer):
         if not isinstance(expr, AppliedUndef):
             self.error('Expecting function')
 
-        scale, shift = scale_shift(expr.args[0], t)    
+        scale, shift = scale_shift(expr.args[0], t)
 
         ssym = sympify(str(s))
 
@@ -91,10 +91,9 @@ class LaplaceTransformer(UnilateralForwardTransformer):
         result = sympify(func).subs(ssym, s / scale) / abs(scale)
 
         if shift != 0:
-            result = result * sym.exp(s * shift / scale)    
+            result = result * sym.exp(s * shift / scale)
         return result
 
-
     def integral(self, expr, t, s):
 
         const, expr = factor_const(expr, t)
@@ -118,17 +117,25 @@ class LaplaceTransformer(UnilateralForwardTransformer):
             expr2.args[0] == t - var and limits[0] == 0 and limits[1] == sym.oo):
             return const2 * self.term(expr2.subs(t - var, t), t, s) / s
 
+        # Handle integral(x(tau), (tau, -oo, t))
+        if not limits[0].is_positive and limits[1] == t:
+            if isinstance(expr2, AppliedUndef):
+                if expr2.args[0] == expr.args[1][0]:
+                    return const2 * self.func(expr2, expr2.args[0], s) / s
+
         # Look for convolution integral
-        # TODO, handle convolution with causal functions.
-        if (limits[0] != -sym.oo) or (limits[1] != sym.oo):
-            self.error('Need indefinite limits')
+        if limits[0].is_positive:
+            self.error('Cannot handle lower limit %s' % limits[0])
+
+        if limits[1] < t:
+            self.error('Cannot handle upper limit %s' % limits[1])
 
         if ((len(expr.args) != 2) or not expr2.is_Mul or
             not expr2.args[0].is_Function or not expr2.args[1].is_Function):
             self.error('Need integral of product of two functions')
 
         f1 = expr2.args[0]
-        f2 = expr2.args[1]    
+        f2 = expr2.args[1]
         # TODO: apply similarity theorem if have f(a * tau) etc.
 
         if (f1.args[0] == var and f2.args[0] == t - var):
@@ -137,7 +144,7 @@ class LaplaceTransformer(UnilateralForwardTransformer):
         elif (f2.args[0] == var and f1.args[0] == t - var):
             F1 = self.term(f1.subs(t - var, t), t, s)
             F2 = self.term(f2, var, s)
-        else:            
+        else:
             self.error('Cannot recognise convolution')
 
         return const2 * F1 * F2
@@ -151,26 +158,75 @@ class LaplaceTransformer(UnilateralForwardTransformer):
             expr.args[1][0] != t):
             self.error('Expecting function of t')
 
-        ssym = sympify(str(s))    
-        name = expr.args[0].func.__name__    
-        func1 = name[0].upper() + name[1:] + '(%s)' % str(ssym)    
+        ssym = sympify(str(s))
+        name = expr.args[0].func.__name__
+        func1 = name[0].upper() + name[1:] + '(%s)' % str(ssym)
         return sympify(func1).subs(ssym, s) * s ** expr.args[1][1]
 
     def sin_cos(self, expr, t, s):
 
+        # Handle exp(-alpha * t) * sin(omega * t + phi) * u(t - tau)
+        # The exp(-alpha * t) and u(t - tau) parts are optional.
+
         # Sympy sometimes has problems with this...
 
-        if not expr.is_Function or expr.func not in (sym.sin, sym.cos):
-            self.error('Expression not sin or cos')
-        arg = expr.args[0]
-        a, b = scale_shift(arg, t)
+        factors = expr.as_ordered_factors()
 
-        d = s**2 + a**2
+        if len(factors) > 3:
+            raise ValueError('Not expsin, too many factors')
 
-        if expr.func is sym.sin:
-            return (a * sym.cos(b) + s * sym.sin(b)) / d
-        else:
-            return (-a * sym.sin(b) + s * sym.cos(b)) / d        
+        alpha = 0
+        beta = 0
+        m = 0
+        if (factors[m].is_Function and factors[m].func is sym.exp):
+            exparg = factors[m].args[0]
+            alpha, beta = scale_shift(exparg, t)
+            m += 1
+
+        if not (factors[m].is_Function and factors[m].func in (sym.sin, sym.cos)):
+            raise ValueError('Not expsin, no sin/cos')
+
+        sincosarg = factors[m].args[0]
+        omega, phi = scale_shift(sincosarg, t)
+
+        if factors[m].func is sym.cos:
+            phi += sym.pi / 2
+
+        m += 1
+
+        if len(factors) == m + 1 and not (factors[m].is_Function and factors[m].func is sym.Heaviside):
+            raise ValueError('Not expsin, no Heaviside')
+
+        tau = 0
+        if len(factors) == m + 1:
+            eta, zeta = scale_shift(factors[m].args[0], t)
+            if eta != 1:
+                raise ValueError('Need to use similarity theorem')
+            tau = -zeta
+
+            if tau.is_negative:
+                tau = 0
+            elif tau.is_Symbol and not tau.is_positive:
+                print('Assuming %s is positive' % tau)
+            elif (tau.is_Mul and len(tau.args) == 2 and
+                  tau.args[0].is_negative and tau.args[1].is_Symbol):
+                print('Assuming %s is positive' % -tau)
+                tau = 0
+
+        if tau != 0:
+            phi += omega * tau
+
+        E = (omega * sym.cos(phi) + (s - alpha) * sym.sin(phi)) / (omega**2 + (s - alpha)**2)
+
+        if tau != 0:
+            E *= sym.exp(-tau * s)
+            if alpha != 0:
+                E *= sym.exp(alpha * tau)
+
+        if beta != 0:
+            E = sym.exp(beta) * E
+
+        return E
 
     def term(self, expr, t, s):
 
@@ -180,7 +236,7 @@ class LaplaceTransformer(UnilateralForwardTransformer):
 
         const, expr = factor_const(expr, t)
 
-        terms = expr.as_ordered_terms()
+        terms = expr.expand(deep=False).as_ordered_terms()
         if len(terms) > 1:
             result = 0
             for term in terms:
@@ -193,13 +249,16 @@ class LaplaceTransformer(UnilateralForwardTransformer):
         if expr.has(sym.Integral):
             return self.integral(expr, t, s) * const
 
-        if expr.is_Function and expr.func in (sym.sin, sym.cos) and expr.args[0].has(t):
-            return self.sin_cos(expr, t, s) * const
+        if expr.has(sym.sin, sym.cos):
+            try:
+                return self.sin_cos(expr, t, s) * const
+            except:
+                pass
 
         if expr.has(AppliedUndef):
 
             if expr.has(sym.Derivative):
-                return self.derivative_undef(expr, t, s) * const    
+                return self.derivative_undef(expr, t, s) * const
 
             factors = expr.as_ordered_factors()
             if len(factors) == 1:
@@ -210,7 +269,7 @@ class LaplaceTransformer(UnilateralForwardTransformer):
             foo = factors[1]
             if foo.is_Function and foo.func == sym.exp and foo.args[0].has(t):
                 scale, shift = scale_shift(foo.args[0], t)
-                if shift == 0: 
+                if shift == 0:
                     result = self.func(factors[0], t, s)
                     return const * result.subs(s, s - scale)
             self.error('Cannot handle product')
@@ -231,19 +290,18 @@ laplace_transformer = LaplaceTransformer()
 
 
 def laplace_transform(expr, t, s, evaluate=True, **kwargs):
-    """Compute unilateral Laplace transform of expr with lower limit 0-.    
+    """Compute unilateral Laplace transform of expr with lower limit 0-.
     """
-    
+
     return laplace_transformer.transform(expr, t, s,
                                          evaluate=evaluate,
                                          **kwargs)
 
 
 def LT(expr, t, s, evaluate=True, **kwargs):
-    """Compute unilateral Laplace transform of expr with lower limit 0-.    
-    """    
+    """Compute unilateral Laplace transform of expr with lower limit 0-.
+    """
 
     return laplace_transformer.transform(expr, t, s,
                                          evaluate=evaluate,
                                          **kwargs)
-
