@@ -29,11 +29,12 @@ from .admittance import admittance
 from .voltage import voltage
 from .current import current
 from sympy import Derivative, Integral
+from warnings import warn
 
 
 __all__ = ('V', 'I', 'v', 'i', 'R', 'NR', 'L', 'C', 'G', 'Y', 'Z',
            'Vdc', 'Vstep', 'Idc', 'Istep', 'Vac', 'sV', 'sI',
-           'Iac', 'Vnoise', 'Inoise', 
+           'Iac', 'Vnoise', 'Inoise',
            'Par', 'Ser', 'Xtal', 'FerriteBead', 'CPE', 'series', 'parallel',
            'ladder')
 
@@ -82,9 +83,9 @@ class OnePort(Network, ImmittanceMixin):
             return self._Z
         if self._Y is not None:
             return 1 / self._Y
-        if self._Voc is not None:        
+        if self._Voc is not None:
             return impedance(0)
-        if self._Isc is not None:        
+        if self._Isc is not None:
             return 1 / admittance(0)
         raise ValueError('_Isc, _Voc, _Y, or _Z undefined for %s' % self)
 
@@ -96,20 +97,18 @@ class OnePort(Network, ImmittanceMixin):
 
     @property
     def Voc(self):
-        """Open-circuit voltage."""        
+        """Open-circuit voltage."""
         if self._Voc is not None:
             return self._Voc
         if self._Isc is not None:
             return self._Isc._mul(self.impedance)
-        if self._Z is not None:        
+        if self._Z is not None or self._Y is not None:
             return SuperpositionVoltage(0)
-        if self._Y is not None:        
-            return SuperpositionCurrent(0)
         raise ValueError('_Isc, _Voc, _Y, or _Z undefined for %s' % self)
-    
+
     @property
     def Isc(self):
-        """Short-circuit current."""        
+        """Short-circuit current."""
         if self._Isc is not None:
             return self._Isc
         return self.Voc._mul(self.admittance)
@@ -144,10 +143,22 @@ class OnePort(Network, ImmittanceMixin):
     def has_series_I(self):
         return self.is_current_source
 
-    @property    
+    @property
     def has_parallel_V(self):
         return self.is_voltage_source
-    
+
+    def chain(self, TP):
+        """Chain to a two-port.  This is experimental."""
+
+        if isinstance(TP, OnePort):
+            raise ValueError('Cannot chain oneport with a oneport')
+
+        from .twoport import TwoPort
+        if not isinstance(TP, TwoPort):
+            raise ValueError('%s not a twoport' % TP)
+
+        return TP.source(self)
+
     def series(self, OP):
         """Series combination"""
 
@@ -192,7 +203,7 @@ class OnePort(Network, ImmittanceMixin):
             raise TypeError('Load argument not ', OnePort)
 
         return LoadCircuit(self, OP2)
-    
+
     @property
     def voc(self):
         """Open-circuit time-domain voltage."""
@@ -200,7 +211,7 @@ class OnePort(Network, ImmittanceMixin):
 
     @property
     def isc(self):
-        """Short-circuit time-domain current."""        
+        """Short-circuit time-domain current."""
         return self.Isc.time()
 
     @property
@@ -215,7 +226,7 @@ class OnePort(Network, ImmittanceMixin):
 
     @property
     def y(self):
-        """Admittance impulse-response."""        
+        """Admittance impulse-response."""
         return self.admittance.time()
 
     def thevenin(self):
@@ -226,8 +237,7 @@ class OnePort(Network, ImmittanceMixin):
         Z = new.impedance
 
         if Voc.is_superposition and not Z.is_real:
-            print('Warning, detected superposition with reactive impedance,'
-                  ' using s-domain.')
+            warn('Detected superposition with reactive impedance, using s-domain.')
             Z1 = Z
             V1 = Voc.laplace()
         elif Voc.is_ac:
@@ -241,7 +251,7 @@ class OnePort(Network, ImmittanceMixin):
             Z1 = Z
 
         V1 = V1.cpt()
-        Z1 = Z1.cpt()        
+        Z1 = Z1.cpt()
 
         if Voc == 0:
             return Z1
@@ -256,10 +266,9 @@ class OnePort(Network, ImmittanceMixin):
         new = self.simplify()
         Isc = new.Isc
         Y = new.admittance
-        
+
         if Isc.is_superposition and not Y.is_real:
-            print('Warning, detected superposition with reactive impedance,'
-                  ' using s-domain.')
+            warn('Detected superposition with reactive impedance, using s-domain.')
             Y1 = Y
             I1 = Isc.laplace()
         elif Isc.is_ac:
@@ -273,8 +282,8 @@ class OnePort(Network, ImmittanceMixin):
             Y1 = Y
 
         I1 = I1.cpt()
-        Y1 = Y1.cpt()        
-            
+        Y1 = Y1.cpt()
+
         if Isc == 0:
             return Y1
         if Y == 0:
@@ -302,7 +311,7 @@ class OnePort(Network, ImmittanceMixin):
         elif self._Z is not None:
             return Z(self._Z)
         elif self._Y is not None:
-            return Y(self._Y)        
+            return Y(self._Y)
         raise RuntimeError('Internal error')
 
     def noise_model(self):
@@ -312,7 +321,7 @@ class OnePort(Network, ImmittanceMixin):
 
         if not isinstance(self, (R, G, Y, Z)):
             return self
-        
+
         R1 = self.R
         if R1 != 0:
             Vn = Vnoise('sqrt(4 * k_B * T * %s)' % R1(j * omega))
@@ -320,14 +329,32 @@ class OnePort(Network, ImmittanceMixin):
         return self
 
         def i_equation(self, v, kind='t'):
-            
+
             raise NotImplementedError('i_equation not defined')
 
         def v_equation(self, i, kind='t'):
-            
-            raise NotImplementedError('v_equation not defined')        
-        
-    
+
+            raise NotImplementedError('v_equation not defined')
+
+    def _Zkind(self, kind):
+
+        # This is for determining impedances
+        if not isinstance(kind, str):
+            # AC
+            domain = kind
+        elif kind in ('super', 'time', 't'):
+            domain = 't'
+        elif kind in ('laplace', 'ivp', 's'):
+            domain = 's'
+        elif kind == 'dc':
+            domain = 0
+        elif kind.startswith('n'):
+            domain = 'f'
+        else:
+            raise RuntimeError('Unhandled circuit kind ' + kind)
+        return self.Z(domain)
+
+
 class ParSer(OnePort):
     """Parallel/serial class"""
 
@@ -577,7 +604,7 @@ class Par(ParSer):
 
         if len(args) < 2:
             raise ValueError('Par requires at least two args')
-        
+
         _check_oneport_args(args)
         super(Par, self).__init__()
         self.args = args
@@ -620,12 +647,12 @@ class Par(ParSer):
         n3, n4 =  netlist._node, netlist._node
 
         H = [(arg.height + self.hsep) * 0.5 for arg in self.args]
-        
+
         N = len(H)
         num_branches = N // 2
 
         # Draw component in centre if have odd number in parallel.
-        if (N & 1):
+        if N & 1:
             s.append(self.args[N // 2]._net_make(netlist, n3, n4, dir))
 
         na, nb = n3, n4
@@ -673,9 +700,9 @@ class Par(ParSer):
         s.append('W %s %s; %s=%s' % (n4, n2, dir, self.wsep))
         return '\n'.join(s)
 
-    @property    
+    @property
     def has_parallel_V(self):
-        
+
         for cpt1 in self.args:
             if cpt1.has_parallel_V:
                 return True
@@ -692,17 +719,25 @@ class Par(ParSer):
     def impedance(self):
         return 1 / self.admittance
 
+    @property
+    def Isc(self):
+        I = 0
+        for arg in self.args:
+            I += arg.Isc
+        return I
+
+
 class Ser(ParSer):
     """Series class"""
 
     _operator = '+'
-    is_series = True    
+    is_series = True
 
     def __init__(self, *args):
 
         if len(args) < 2:
             raise ValueError('Ser requires at least two args')
-        
+
         _check_oneport_args(args)
         super(Ser, self).__init__()
         self.args = args
@@ -716,7 +751,7 @@ class Ser(ParSer):
                     print('Warn: redundant component %s in series with current source %s' % (arg2, arg1))
 
                 elif isinstance(arg2, I):
-                    print('Warn: redundant component %s in series with current source %s' % (arg1, arg2))                    
+                    print('Warn: redundant component %s in series with current source %s' % (arg1, arg2))
 
     @property
     def height(self):
@@ -752,17 +787,17 @@ class Ser(ParSer):
         s.append(self.args[-1]._net_make(netlist, n1, n2, dir))
         return '\n'.join(s)
 
-    @property    
+    @property
     def has_series_I(self):
         for cpt1 in self.args:
-            if cpt1.has_series_I:            
+            if cpt1.has_series_I:
                 return True
         return False
-    
+
     @property
     def admittance(self):
         return 1 / self.impedance
-    
+
     @property
     def impedance(self):
         Z = 0
@@ -770,15 +805,23 @@ class Ser(ParSer):
             Z += arg.impedance
         return Z
 
-    
+    @property
+    def Voc(self):
+        V = 0
+        for arg in self.args:
+            V += arg.Voc
+        return V
+
+
 class R(OnePort):
     """Resistor"""
 
     is_resistor = True
     is_noiseless = False
-    
-    def __init__(self, Rval='R'):
 
+    def __init__(self, Rval='R', **kwargs):
+
+        self.kwargs = kwargs
         self.args = (Rval, )
         self._R = cexpr(Rval)
         self._Z = impedance(self._R, causal=True)
@@ -802,10 +845,11 @@ class G(OnePort):
     """Conductor"""
 
     is_conductor = True
-    is_noiseless = False    
+    is_noiseless = False
 
-    def __init__(self, Gval='G'):
+    def __init__(self, Gval='G', **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Gval, )
         self._G = cexpr(Gval)
         self._Z = impedance(1 / self._G, causal=True)
@@ -816,7 +860,8 @@ class G(OnePort):
             n1 = netlist._node
         if n2 == None:
             n2 = netlist._node
-        return 'R? %s %s {%s}; %s' % (n1, n2, 1 / self._G, dir)
+        opts_str = self._opts_str(dir)
+        return 'R? %s %s {%s}; %s' % (n1, n2, 1 / self._G, opts_str)
 
     def i_equation(self, v, kind='t'):
 
@@ -825,14 +870,14 @@ class G(OnePort):
     def v_equation(self, i, kind='t'):
 
         return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z).select(kind)
-    
+
 
 class NG(G):
     """Noiseless conductor"""
 
     is_noiseless = True
-    
-    
+
+
 class L(OnePort):
     """Inductor
 
@@ -840,8 +885,9 @@ class L(OnePort):
 
     is_inductor = True
 
-    def __init__(self, Lval='L', i0=None):
+    def __init__(self, Lval='L', i0=None, **kwargs):
 
+        self.kwargs = kwargs
         self.has_ic = i0 is not None
         if i0 is None:
             i0 = 0
@@ -857,7 +903,7 @@ class L(OnePort):
         self.i0 = i0
         self._Z = impedance(s * Lval, causal=True)
         self._Voc = SuperpositionVoltage(LaplaceDomainExpression(-i0 * Lval))
-        self.zeroic = self.i0 == 0 
+        self.zeroic = self.i0 == 0
 
     def i_equation(self, v, kind='t'):
 
@@ -867,24 +913,25 @@ class L(OnePort):
             u = tausym
             v = expr(v).subs(t, u)
             return SuperpositionCurrent(expr(Integral(v.expr, (u, -oo, tsym))) / self.L).select(kind)
-        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self.Z(kind)).select(kind)
+        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self._Zkind(kind)).select(kind)
 
     def v_equation(self, i, kind='t'):
 
         if kind in ('t', 'time', 'super'):
             return SuperpositionVoltage(self.L * expr(Derivative(i.expr, t))).select(kind)
-        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z(kind)).select(kind)
+        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Zkind(kind)).select(kind)
+
 
-    
 class C(OnePort):
     """Capacitor
 
     Capacitance Cval, initial voltage v0"""
 
-    is_capacitor = True    
+    is_capacitor = True
 
-    def __init__(self, Cval='C', v0=None):
+    def __init__(self, Cval='C', v0=None, **kwargs):
 
+        self.kwargs = kwargs
         self.has_ic = v0 is not None
         if v0 is None:
             v0 = 0
@@ -903,10 +950,10 @@ class C(OnePort):
         self.zeroic = self.v0 == 0
 
     def i_equation(self, v, kind='t'):
-        
+
         if kind in ('t', 'time', 'super'):
             return SuperpositionCurrent(self.C * expr(Derivative(v.expr, t))).select(kind)
-        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self._Z(kind)).select(kind)
+        return SuperpositionCurrent(SuperpositionVoltage(v).select(kind) / self._Zkind(kind)).select(kind)
 
     def v_equation(self, i, kind='t'):
 
@@ -916,7 +963,7 @@ class C(OnePort):
             u = tausym
             i = expr(i).subs(t, u)
             return SuperpositionVoltage(expr(Integral(i.expr, (u, -oo, tsym))) / self.C).select(kind)
-        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Z(kind)).select(kind)
+        return SuperpositionVoltage(SuperpositionCurrent(i).select(kind) * self._Zkind(kind)).select(kind)
 
 
 class CPE(OnePort):
@@ -934,8 +981,9 @@ class CPE(OnePort):
     as a rational function and so there are no poles or zeros.  So
     don't be suprised if Lcapy throws an occasional wobbly."""
 
-    def __init__(self, K, alpha=0.5):
+    def __init__(self, K, alpha=0.5, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (K, alpha)
 
         K = cexpr(K)
@@ -948,8 +996,9 @@ class CPE(OnePort):
 class Y(OnePort):
     """General admittance."""
 
-    def __init__(self, Yval):
+    def __init__(self, Yval='Y', **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Yval, )
         Yval = admittance(Yval)
         self._Y = Yval
@@ -958,8 +1007,9 @@ class Y(OnePort):
 class Z(OnePort):
     """General impedance."""
 
-    def __init__(self, Zval):
+    def __init__(self, Zval='Z', **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Zval, )
         Zval = impedance(Zval)
         self._Z = Zval
@@ -972,17 +1022,18 @@ class VoltageSourceBase(OnePort):
     is_noisy = False
 
     def v_equation(self, i, kind='t'):
-        
+
         return SuperpositionVoltage(self.voc).select(kind)
-    
+
 
 class sV(VoltageSourceBase):
     """Arbitrary s-domain voltage source"""
 
     netkeyword = 's'
 
-    def __init__(self, Vval):
+    def __init__(self, Vval, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Vval, )
         Vval = LaplaceDomainExpression(Vval)
         self._Voc = SuperpositionVoltage(LaplaceDomainExpression(Vval))
@@ -991,19 +1042,21 @@ class sV(VoltageSourceBase):
 class V(VoltageSourceBase):
     """Arbitrary voltage source"""
 
-    def __init__(self, Vval='V'):
+    def __init__(self, Vval='V', **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Vval, )
         self._Voc = SuperpositionVoltage(Vval)
 
-        
+
 class Vstep(VoltageSourceBase):
     """Step voltage source (s domain voltage of v / s)."""
 
     netkeyword = 'step'
 
-    def __init__(self, v):
+    def __init__(self, v, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (v, )
         v = cexpr(v)
         self._Voc = SuperpositionVoltage(TimeDomainExpression(v) * Heaviside(t))
@@ -1015,9 +1068,10 @@ class Vdc(VoltageSourceBase):
     an s domain voltage of V / s)."""
 
     netkeyword = 'dc'
-    
-    def __init__(self, v):
 
+    def __init__(self, v, **kwargs):
+
+        self.kwargs = kwargs
         self.args = (v, )
         v = cexpr(v)
         self._Voc = SuperpositionVoltage(cexpr(v, dc=True))
@@ -1033,20 +1087,21 @@ class Vac(VoltageSourceBase):
 
     netkeyword = 'ac'
 
-    def __init__(self, V, phi=None, omega=None):
+    def __init__(self, V, phi=None, omega=None, **kwargs):
 
+        self.kwargs = kwargs
         if phi is None and omega is None:
             self.args = (V, )
         elif phi is not None and omega is None:
             self.args = (V, phi)
         elif phi is None and omega is not None:
-            self.args = (V, 0, omega)            
+            self.args = (V, 0, omega)
         else:
-            self.args = (V, phi, omega)            
+            self.args = (V, phi, omega)
 
         if phi is None:
             phi = 0
-            
+
         if omega is None:
             omega = omega0sym
         omega = expr(omega)
@@ -1073,18 +1128,20 @@ class Vnoise(VoltageSourceBase):
     netkeyword = 'noise'
     is_noisy = True
 
-    def __init__(self, V, nid=None):
+    def __init__(self, V, nid=None, **kwargs):
 
+        self.kwargs = kwargs
         V1 = AngularFourierNoiseDomainVoltage(V, nid=nid)
         self.args = (V, V1.nid)
         self._Voc = SuperpositionVoltage(V1)
 
-        
+
 class v(VoltageSourceBase):
     """Arbitrary t-domain voltage source"""
 
-    def __init__(self, vval):
+    def __init__(self, vval, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (vval, )
         Vval = TimeDomainExpression(vval)
         self._Voc = SuperpositionVoltage(Vval)
@@ -1094,7 +1151,7 @@ class CurrentSourceBase(OnePort):
 
     is_current_source = True
     netname = 'I'
-    is_noisy = False    
+    is_noisy = False
 
     @property
     def I(self):
@@ -1104,16 +1161,17 @@ class CurrentSourceBase(OnePort):
 
     def i_equation(self, v, kind='t'):
 
-        return SuperpositionCurrent(self.isc).select(kind)        
-    
-    
+        return SuperpositionCurrent(self.isc).select(kind)
+
+
 class sI(CurrentSourceBase):
     """Arbitrary s-domain current source"""
 
     netkeyword = 's'
 
-    def __init__(self, Ival):
+    def __init__(self, Ival, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Ival, )
         Ival = LaplaceDomainExpression(Ival)
         self._Isc = SuperpositionCurrent(LaplaceDomainExpression(Ival))
@@ -1122,19 +1180,21 @@ class sI(CurrentSourceBase):
 class I(CurrentSourceBase):
     """Arbitrary current source"""
 
-    def __init__(self, Ival='I'):
+    def __init__(self, Ival='I', **kwargs):
 
+        self.kwargs = kwargs
         self.args = (Ival, )
         self._Isc = SuperpositionCurrent(Ival)
 
-            
+
 class Istep(CurrentSourceBase):
     """Step current source (s domain current of i / s)."""
 
     netkeyword = 'step'
 
-    def __init__(self, i):
+    def __init__(self, i, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (i, )
         i = cexpr(i)
         self._Isc = SuperpositionCurrent(TimeDomainExpression(i) * Heaviside(t))
@@ -1146,9 +1206,10 @@ class Idc(CurrentSourceBase):
     an s domain current of i / s)."""
 
     netkeyword = 'dc'
-    
-    def __init__(self, i):
 
+    def __init__(self, i, **kwargs):
+
+        self.kwargs = kwargs
         self.args = (i, )
         i = cexpr(i)
         self._Isc = SuperpositionCurrent(cexpr(i, dc=True))
@@ -1164,20 +1225,21 @@ class Iac(CurrentSourceBase):
 
     netkeyword = 'ac'
 
-    def __init__(self, I, phi=0, omega=None):
+    def __init__(self, I, phi=0, omega=None, **kwargs):
 
+        self.kwargs = kwargs
         if phi is None and omega is None:
             self.args = (I, )
         elif phi is not None and omega is None:
             self.args = (I, phi)
         elif phi is None and omega is not None:
-            self.args = (I, 0, omega)            
+            self.args = (I, 0, omega)
         else:
-            self.args = (I, phi, omega)            
+            self.args = (I, phi, omega)
 
         if phi is None:
             phi = 0
-            
+
         if omega is None:
             omega = omega0sym
         omega = cexpr(omega)
@@ -1202,18 +1264,20 @@ class Inoise(CurrentSourceBase):
     netkeyword = 'noise'
     is_noisy = True
 
-    def __init__(self, I, nid=None):
+    def __init__(self, I, nid=None, **kwargs):
 
+        self.kwargs = kwargs
         I1 = AngularFourierNoiseDomainCurrent(I, nid=nid)
         self._Isc = SuperpositionCurrent(I1)
         self.args = (I, I1.nid)
 
-        
+
 class i(CurrentSourceBase):
     """Arbitrary t-domain current source"""
 
-    def __init__(self, ival):
+    def __init__(self, ival, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (ival, )
         Ival = TimeDomainExpression(ival)
         self._Isc = SuperpositionCurrent(Ival)
@@ -1227,8 +1291,9 @@ class Xtal(OnePort):
     harmonic resonances are not modelled.
     """
 
-    def __init__(self, C0, R1, L1, C1):
+    def __init__(self, C0, R1, L1, C1, **kwargs):
 
+        self.kwargs = kwargs
         self.C0 = cexpr(C0)
         self.R1 = cexpr(R1)
         self.L1 = cexpr(L1)
@@ -1245,7 +1310,7 @@ class Xtal(OnePort):
 
         # TODO: draw this with a symbol
         net = self.expand()
-        return net._net_make(netlist, n1, n2, dir)    
+        return net._net_make(netlist, n1, n2, dir)
 
 
 class FerriteBead(OnePort):
@@ -1255,8 +1320,9 @@ class FerriteBead(OnePort):
     to a parallel R, L, C network (Rp, Lp, Cp).
     """
 
-    def __init__(self, Rs, Rp, Cp, Lp):
+    def __init__(self, Rs, Rp, Cp, Lp, **kwargs):
 
+        self.kwargs = kwargs
         self.Rs = cexpr(Rs)
         self.Rp = cexpr(Rp)
         self.Cp = cexpr(Cp)
@@ -1273,8 +1339,9 @@ class FerriteBead(OnePort):
 
         # TODO: draw this with a symbol
         net = self.expand()
-        return net._net_make(netlist, n1, n2, dir)            
-    
+        return net._net_make(netlist, n1, n2, dir)
+
+
 class LoadCircuit(Network):
     """Circuit comprised of a load oneport connected in parallel with a
     source oneport."""
@@ -1321,52 +1388,58 @@ class ControlledSource(OnePort):
 
 class CCVS(ControlledSource):
 
-    def __init__(self, control, value):
+    def __init__(self, control, value, **kwargs):
 
-        self.args = (control, value)        
+        self.kwargs = kwargs
+        self.args = (control, value)
         self._Voc = SuperpositionVoltage(0)
         self._Z = impedance(0)
 
-        
+
 class CCCS(ControlledSource):
 
-    def __init__(self, control, value):
+    def __init__(self, control, value, **kwargs):
 
-        self.args = (control, value)        
+        self.kwargs = kwargs
+        self.args = (control, value)
         self._Isc = SuperpositionCurrent(0)
-        self._Y = admittance(0)     
+        self._Y = admittance(0)
+
 
-        
 class VCVS(ControlledSource):
 
-    def __init__(self, value):
+    def __init__(self, value, **kwargs):
 
-        self.args = (value, )        
+        self.kwargs = kwargs
+        self.args = (value, )
         self._Voc = SuperpositionVoltage(0)
         self._Z = impedance(0)
 
-        
+
 class VCCS(ControlledSource):
 
-    def __init__(self, value):
+    def __init__(self, value, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (value, )
         self._Isc = SuperpositionCurrent(0)
-        self._Y = admittance(0)     
-        
+        self._Y = admittance(0)
+
 
 class Dummy(OnePort):
 
-    def __init__(self, *args):
+    def __init__(self, *args, **kwargs):
 
+        self.kwargs = kwargs
         self.args = args
 
 
 class K(Dummy):
     """Coupling coefficient"""
 
-    def __init__(self, L1, L2, K):
+    def __init__(self, L1, L2, K, **kwargs):
 
+        self.kwargs = kwargs
         self.args = (L1, L2, K)
         self.K = cexpr(K)
 
@@ -1374,21 +1447,23 @@ class K(Dummy):
 class W(Dummy):
     """Wire (short)"""
 
-    def __init__(self):
+    def __init__(self, **kwargs):
 
+        self.kwargs = kwargs
         self.args = ()
         self._Z = impedance(0)
 
-        
+
 class O(Dummy):
     """Open circuit"""
 
-    def __init__(self):
+    def __init__(self, **kwargs):
 
+        self.kwargs = kwargs
         self.args = ()
         self._Y = admittance(0)
 
-        
+
 class P(O):
     """Port (open circuit)"""
     pass
@@ -1431,7 +1506,7 @@ def series(*args):
 def parallel(*args):
     """Create a parallel combination of a number of components.
     Args that are None are ignored.  If there is only one
-    non-None component, return that component."""    
+    non-None component, return that component."""
 
     args = [net for net in args if net is not None]
     if args == []:
@@ -1451,7 +1526,7 @@ def ladder(*args, **kwargs):
     """
 
     start_series = kwargs.pop('start_series', True)
-    
+
     if len(args) == 0:
         return None
     elif len(args) == 1:
@@ -1461,7 +1536,7 @@ def ladder(*args, **kwargs):
     else:
         return parallel(args[0], ladder(*args[1:], start_series = not start_series))
 
-    
+
 # Imports at end to circumvent circular dependencies
 from .expr import Expr, expr
 from .cexpr import cexpr
@@ -1472,4 +1547,3 @@ from .noiseomegaexpr import AngularFourierNoiseDomainCurrent, AngularFourierNois
 from .superpositionvoltage import SuperpositionVoltage
 from .superpositioncurrent import SuperpositionCurrent
 from .twoport import Ladder, LSection, TSection
-
