@@ -7,8 +7,12 @@ Copyright 2016--2020 Michael Hayes, UCECE
 from __future__ import division
 import sympy as sym
 from .sym import sympify, AppliedUndef
+from .cache import lru_cache
 from .utils import pair_conjugates
 
+Zero = sym.S.Zero
+One = sym.S.One
+
 class Pole(object):
 
     def __init__(self, expr, n, damping=None):
@@ -21,12 +25,12 @@ class Pole(object):
     def _decompose_bar(self, expr):
 
         # Look for scale * sqrt(dexpr) where dexpr = aexpr - bexpr
-    
+
         dexpr = None
-        scale = sym.S.One
-    
+        scale = One
+
         for factor in expr.as_ordered_factors():
-            if (factor.is_Pow and factor.args[1] == sym.S.One / 2 and
+            if (factor.is_Pow and factor.args[1] == One / 2 and
                 factor.args[0].is_Add and factor.args[0].args[1].is_Mul and
                 factor.args[0].args[1].args[0] < 0):
                 if dexpr is not None:
@@ -41,9 +45,9 @@ class Pole(object):
     def _decompose_foo(self, expr):
 
         # Look for offset + scale * sqrt(dexpr) where dexpr = aexpr - bexpr
-    
-        offset = sym.S.Zero
-        scale = sym.S.One
+
+        offset = Zero
+        scale = One
         found = False
         dexpr = None
 
@@ -60,11 +64,11 @@ class Pole(object):
 
     def _decompose(self, expr):
 
-        scale = sym.S.One
-        scale2 = sym.S.One    
-        offset = sym.S.One
+        scale = One
+        scale2 = One
+        offset = One
         dexpr = None
-    
+
         for factor in expr.as_ordered_factors():
             if factor.is_Add:
                 p = self._decompose_foo(factor)
@@ -78,7 +82,7 @@ class Pole(object):
                 scale *= factor
 
         if dexpr is None:
-            dexpr = sym.S.Zero
+            dexpr = Zero
         return scale, offset, scale2, dexpr
 
     @property
@@ -86,8 +90,8 @@ class Pole(object):
         return self.conjugate()
 
     def conjugate(self):
-        return self._rewrite(conjugate=True)            
-        
+        return self._rewrite(conjugate=True)
+
     def _rewrite(self, conjugate=False):
 
         if self.damping in (None, 'over') or self.d is None:
@@ -110,8 +114,8 @@ class Pole(object):
 
 def as_numer_denom_poly(expr, var):
 
-    N = sym.S.One
-    D = sym.S.One    
+    N = One
+    D = One
     for f in expr.as_ordered_factors():
         if f.is_Pow and f.args[1] == -1:
             D *= f.args[0]
@@ -120,12 +124,12 @@ def as_numer_denom_poly(expr, var):
 
     try:
         Dpoly = sym.Poly(D, var)
-        Npoly = sym.Poly(N, var)        
+        Npoly = sym.Poly(N, var)
     except:
         N, D = expr.as_numer_denom()
         Dpoly = sym.Poly(D, var)
-        Npoly = sym.Poly(N, var)                
-        
+        Npoly = sym.Poly(N, var)
+
     return Npoly, Dpoly
 
 
@@ -133,9 +137,9 @@ def as_numer_denom(expr, var):
 
     if expr.has(1 / var):
         expr = expr.cancel()
-    
-    N = sym.S.One
-    D = sym.S.One    
+
+    N = One
+    D = One
     for f in expr.as_ordered_factors():
         if f.is_Pow and f.args[1] == -1:
             D *= f.args[0]
@@ -169,7 +173,7 @@ def _zp2tf(zeros, poles, K=1, var=None):
         pp = [1 / (var - p) for p in poles]
     else:
         pp = [1 / (var - p) ** poles[p] for p in poles]
-        
+
     return sym.Mul(K, *(zz + pp), evaluate=False)
 
 
@@ -185,7 +189,7 @@ def _tc2tf(zeros, poles, K=1, var=None):
 
     zz = []
     pp = []
-    
+
     if isinstance(zeros, (tuple, list)):
         for z in zeros:
             if z == 0:
@@ -199,7 +203,7 @@ def _tc2tf(zeros, poles, K=1, var=None):
                 zz.append(var ** o)
             else:
                 zz.append((var / -z + 1) ** zeros[z])
-                K *= z ** o       
+                K *= z ** o
 
     if isinstance(zeros, (tuple, list)):
         for p in poles:
@@ -214,7 +218,7 @@ def _tc2tf(zeros, poles, K=1, var=None):
                 pp.append(var ** o)
             else:
                 pp.append(1 / (var / -p + 1) ** o)
-                K /= p ** o        
+                K /= p ** o
 
     K = K.simplify()
     return sym.Mul(K, *(zz + pp), evaluate=False)
@@ -231,7 +235,7 @@ def _pr2tf(poles, residues, var=None):
 
 
 def as_ratfun_delay(expr, var):
-    delay = sym.S.Zero
+    delay = Zero
 
     if expr.is_rational_function(var):
         N, D = expr.as_numer_denom()
@@ -239,7 +243,7 @@ def as_ratfun_delay(expr, var):
 
     F = sym.factor(expr).as_ordered_factors()
 
-    rf = sym.S.One
+    rf = One
     for f in F:
         b, e = f.as_base_exp()
         if b == sym.E and e.is_polynomial(var):
@@ -262,16 +266,16 @@ def as_ratfun_delay(expr, var):
 
 
 def as_ratfun_delay_undef(expr, var):
-    delay = sym.S.Zero
-    undef = sym.S.One
-    
+    delay = Zero
+    undef = One
+
     if expr.is_rational_function(var):
         N, D = as_numer_denom(expr, var)
         return N, D, delay, undef
 
     F = sym.factor(expr).as_ordered_factors()
 
-    rf = sym.S.One
+    rf = One
     for f in F:
         b, e = f.as_base_exp()
         if b == sym.E and e.is_polynomial(var):
@@ -285,13 +289,13 @@ def as_ratfun_delay_undef(expr, var):
         if isinstance(f, AppliedUndef):
             undef *= f
             continue
-                
+
         rf *= f
 
     if not rf.is_rational_function(var):
         raise ValueError('Expression not a product of rational function,'
                          ' exponential, and undefined functions')
-    
+
     N, D = rf.as_numer_denom()
     return N, D, delay, undef
 
@@ -306,7 +310,7 @@ class Ratfun(object):
     def as_ratfun_delay(self):
         """Split expr as (N, D, delay)
         where expr = (N / D) * exp(var * delay)
-        
+
         Note, delay only represents a delay when var is s."""
 
         return as_ratfun_delay(self.expr, self.var)
@@ -317,10 +321,10 @@ class Ratfun(object):
         and where N is a polynomial in var,
         D is a polynomial in var, and undef is the product
         of undefined functions, e.g., V(s).
-        
+
         Note, delay only represents a delay when var is s."""
 
-        return as_ratfun_delay_undef(self.expr, self.var)        
+        return as_ratfun_delay_undef(self.expr, self.var)
 
     def as_const_undef_rest(self):
         """Split expr as (const, undef, rest)
@@ -328,11 +332,11 @@ class Ratfun(object):
 
         expr = self.expr
         var = self.var
-        
-        const = sym.S.One
-        undef = sym.S.One
-        rest = sym.S.One        
-    
+
+        const = One
+        undef = One
+        rest = One
+
         F = sym.factor(expr).as_ordered_factors()
 
         for f in F:
@@ -343,21 +347,24 @@ class Ratfun(object):
                 const *= f
                 continue
             rest *= f
-    
+
         return const, undef, rest
-    
+
+    @lru_cache()
     def roots(self):
         """Return roots of expression as a dictionary
         Note this may not find them all."""
 
         return sym.roots(sym.Poly(self.expr, self.var))
 
+    @lru_cache()
     def zeros(self):
         """Return zeroes of expression as a dictionary
         Note this may not find them all."""
 
-        return Ratfun(self.numerator, self.var).roots()    
+        return Ratfun(self.numerator, self.var).roots()
 
+    @lru_cache()
     def poles(self, damping=None):
         """Return poles of expression as a dictionary of Pole objects.
         Note this may not find all the poles."""
@@ -373,7 +380,7 @@ class Ratfun(object):
                     break
             if pole.n != 0:
                 poles.append(pole)
-        
+
         return poles
 
     def residue(self, pole, poles):
@@ -387,24 +394,24 @@ class Ratfun(object):
         occurrences = []
         for p in poles:
             occurrences += [p.n - 1 if p.expr == pole else p.n]
-        
+
         numer, denom = expr.as_numer_denom()
         Dpoly = sym.Poly(denom, var)
         K = Dpoly.LC()
-        
+
         D = [(var - p.expr) ** o for p, o in zip(poles, occurrences)]
         denom = sym.Mul(K, *D)
 
         # Could calculate all residues simultaneously using
         # system of linear equations.
-        
+
         def method1(numer, denom, var, pole):
-        
+
             d = sym.limit(denom, var, pole)
-            
+
             if d != 0:
                 tmp = (numer / denom).simplify()
-                
+
                 # Sometimes this takes ages...
                 return sym.limit(tmp, var, pole)
 
@@ -416,22 +423,22 @@ class Ratfun(object):
 
         def method2(numer, denom, var, pole):
 
-            d = denom.subs(var, pole)            
+            d = denom.subs(var, pole)
             n = numer.subs(var, pole)
-            
+
             if d != 0:
                 return n / d
 
             ddenom = sym.diff(denom, var)
             return n / ddenom.subs(pole)
 
-        m2 = method2(numer, denom, var, pole)        
+        m2 = method2(numer, denom, var, pole)
         return m2.cancel()
 
     @property
     def numerator_denominator(self):
         """Return numerator and denominator of rational function"""
-        
+
         return as_numer_denom(self.expr, self.var)
 
     @property
@@ -441,7 +448,7 @@ class Ratfun(object):
     @property
     def D(self):
         return self.denominator
-    
+
     @property
     def numerator(self):
         """Return numerator of rational function"""
@@ -469,7 +476,7 @@ class Ratfun(object):
 
         See also general, partfrac, standard, timeconst, and ZPK
 
-        """        
+        """
 
         try:
             N, D, delay, undef = self.as_ratfun_delay_undef()
@@ -477,7 +484,7 @@ class Ratfun(object):
             # TODO: copy?
             return self.expr
 
-        var = self.var        
+        var = self.var
         Dpoly = sym.Poly(D, var)
         Npoly = sym.Poly(N, var)
 
@@ -493,9 +500,11 @@ class Ratfun(object):
             if D == 1:
                 expr = N
             else:
-                expr = sym.Mul(N, 1 / D, evaluate=False)                
+                expr = sym.Mul(N, 1 / D, evaluate=False)
 
-            expr = sym.Mul(K, expr, undef, evaluate=False)
+            if K != 1:
+                expr = sym.Mul(K, expr, evaluate=False)
+            expr *= undef
         else:
             C = Dpoly.LC()
             D = Dpoly.monic().as_expr()
@@ -503,11 +512,14 @@ class Ratfun(object):
             if D == 1:
                 expr = N
             else:
-                expr = sym.Mul(N, 1 / D, evaluate=False)
+                if N == 1:
+                    expr = 1 / D
+                else:
+                    expr = sym.Mul(N, 1 / D, evaluate=False)
             if delay != 0:
                 expr *= sym.exp(self.var * delay)
             expr *= undef
-            
+
         return expr
 
     def general(self):
@@ -525,13 +537,13 @@ class Ratfun(object):
 
     def expandcanonical(self):
         """Expand in terms for different powers with each term
-        expressed in canonical form."""        
+        expressed in canonical form."""
 
         N, D, delay, undef = self.as_ratfun_delay_undef()
 
         Npoly = sym.Poly(N, self.var)
-        
-        expr = sym.S.Zero
+
+        expr = Zero
 
         for m, c in enumerate(reversed(Npoly.all_coeffs())):
             term = sym.Mul(c.simplify() * self.var ** m, 1 / D)
@@ -551,19 +563,19 @@ class Ratfun(object):
         See also canonical, standard, general, timeconst, and ZPK
 
         """
-        
+
         try:
             Q, R, D, delay, undef = self.as_QRD(combine_conjugates, damping)
         except ValueError:
             if not split:
                 raise
-            
+
             # Try splitting into terms
             result = 0
             for term in self.expr.as_ordered_terms():
                 result += Ratfun(term, self.var).partfrac(combine_conjugates,
                                                           split=False)
-            return result           
+            return result
 
         result = Q
         for R, D in zip(R, D):
@@ -588,13 +600,13 @@ class Ratfun(object):
         except ValueError:
             if not split:
                 raise
-            
+
             # Try splitting into terms
             result = 0
             for term in self.expr.as_ordered_terms():
                 result += Ratfun(term, self.var).standard(False)
-            return result                       
-        
+            return result
+
         expr = Q + sym.cancel(M / D, self.var)
 
         if delay != 0:
@@ -610,10 +622,10 @@ class Ratfun(object):
 
         N, D, delay, undef = self.as_ratfun_delay_undef()
 
-        var = self.var        
+        var = self.var
         Npoly = sym.Poly(N, var)
         Dpoly = sym.Poly(D, var)
-        
+
         K = Dpoly.EC()
 
         D = D / K
@@ -628,7 +640,7 @@ class Ratfun(object):
         zeros, poles, K, undef = self.as_ZPK()
 
         var = self.var
-        
+
         if not combine_conjugates:
             return _zp2tf(zeros, poles, K, var) * undef
 
@@ -650,9 +662,9 @@ class Ratfun(object):
 
         result2 = _zp2tf(zero_singles, pole_singles, 1, var) * undef
         result = K * result1 * result2
-        
+
         return result
-        
+
     def residues(self, combine_conjugates=False, damping=None):
         """Return residues of partial fraction expansion.
 
@@ -666,7 +678,7 @@ class Ratfun(object):
 
         N, D, delay, undef = self.as_ratfun_delay_undef()
 
-        var = self.var        
+        var = self.var
         Npoly = sym.Poly(N, var)
         Dpoly = sym.Poly(D, var)
 
@@ -678,7 +690,7 @@ class Ratfun(object):
 
         This the maximum of the numerator and denominator degrees.
         Note zero has a degree of -inf."""
-        
+
         Npoly, Dpoly = as_numer_denom_poly(self.expr, self.var)
         return max(Npoly.degree(), Dpoly.degree())
 
@@ -686,7 +698,7 @@ class Ratfun(object):
     def Ndegree(self):
         """Return the degree (order) of the numerator of a rational function.
         Note zero has a degree of -inf."""
-        
+
         Npoly, Dpoly = as_numer_denom_poly(self.expr, self.var)
         return Npoly.degree()
 
@@ -694,10 +706,10 @@ class Ratfun(object):
     def Ddegree(self):
         """Return the degree (order) of the denominator of a rational function.
         Note zero has a degree of -inf."""
-        
+
         Npoly, Dpoly = as_numer_denom_poly(self.expr, self.var)
-        return Dpoly.degree()    
-    
+        return Dpoly.degree()
+
     @property
     def is_strictly_proper(self):
         """Return True if the degree of the dominator is greater
@@ -710,13 +722,13 @@ class Ratfun(object):
 
         expression = K * (prod_n (var - z_n) / (prod_n (var - p_n)) * undef
         """
-                
+
         N, D, delay, undef = self.as_ratfun_delay_undef()
 
-        var = self.var        
+        var = self.var
         Npoly = sym.Poly(N, var)
         Dpoly = sym.Poly(D, var)
-        
+
         K = sym.cancel(Npoly.LC() / Dpoly.LC())
         if delay != 0:
             K *= sym.exp(self.var * delay)
@@ -725,7 +737,7 @@ class Ratfun(object):
         poles = sym.roots(Dpoly)
 
         return zeros, poles, K, undef
-    
+
     def as_QMD(self):
         """Decompose expression into Q, M, D, delay, undef where
 
@@ -737,8 +749,8 @@ class Ratfun(object):
         Q, M = sym.div(N, D, self.var)
 
         return Q, M, D, delay, undef
-        
-    def as_QRD(self, combine_conjugates=False, damping=None):
+
+    def as_QRD_old(self, combine_conjugates=False, damping=None):
         """Decompose expression into Q, R, D, delay, undef where
 
         expression = (Q + sum_n R_n / D_n) * exp(-delay * var) * undef"""
@@ -747,7 +759,7 @@ class Ratfun(object):
 
         expr = M / D
         var = self.var
-        
+
         sexpr = Ratfun(expr, var)
         poles = sexpr.poles(damping=damping)
 
@@ -756,7 +768,7 @@ class Ratfun(object):
             # these variables do not occur in numerator of expr
             # perhaps the following code will work.
             raise ValueError('Critical damping not supported')
-        
+
         polesdict = {}
         for pole in poles:
             polesdict[pole.expr] = pole.n
@@ -765,7 +777,7 @@ class Ratfun(object):
         D = []
 
         for pole in poles:
-        
+
             # Number of occurrences of the pole.
             o = polesdict[pole.expr]
             if o == 0:
@@ -777,9 +789,9 @@ class Ratfun(object):
                 polesdict[pc] -= 1
 
                 D2 = sym.simplify(var**2 - (p + pc) * var + p * pc)
-                    
+
                 if o == 1:
-                    r = sexpr.residue(p, poles)                    
+                    r = sexpr.residue(p, poles)
                     rc = sexpr.residue(pc, poles)
 
                     r = sym.simplify(r * (var - pc) + rc * (var - p))
@@ -787,13 +799,13 @@ class Ratfun(object):
                     D.append(D2)
                 else:
                     # Handle repeated complex pole pairs.
-                    expr2 = expr * (var - p) ** o                
+                    expr2 = expr * (var - p) ** o
                     for n in range(1, o + 1):
                         m = o - n
 
                         dexpr = sym.diff(expr2, var, m)
                         r = sym.limit(dexpr, var, p) / sym.factorial(m)
-                        
+
                         rc = r.conjugate()
                         r = sym.simplify(r * (var - pc) ** n + rc * (var - p) ** n)
                         R.append(r)
@@ -820,7 +832,140 @@ class Ratfun(object):
                         r = sym.limit(dexpr, var, p) / sym.factorial(m)
 
                         R.append(r)
-                        D.append(D2 ** n)                        
-                                   
+                        D.append(D2 ** n)
+
+        return Q, R, D, delay, undef
+
+    def _prune_zero_residues(self, R, P, O):
+
+        Rnew = []
+        Pnew = []
+        Onew = []
+        for r, p, o in zip(R, P, O):
+            if r != 0:
+                Rnew.append(r)
+                Pnew.append(p)
+                Onew.append(o)
+        return Rnew, Pnew, Onew
+
+    def as_QRPO(self, damping=None):
+        """Decompose expression into Q, R, P, O, delay, undef where
+
+        expression = (Q + sum_n R_n / (var - P_n)**O_n) * exp(-delay * var) * undef
+        """
+
+        from .matrix import matrix_inverse
+
+        Q, M, D, delay, undef = self.as_QMD()
+
+        expr = M / D
+        var = self.var
+
+        sexpr = Ratfun(expr, var)
+        poles = sexpr.poles(damping=damping)
+
+        if damping == 'critical':
+            # Critical damping puts constraints on variables.  If
+            # these variables do not occur in numerator of expr
+            # perhaps the following code will work.
+            raise ValueError('Critical damping not supported')
+
+        if len(poles) == 0:
+            return Q, [], [], [], delay, undef
+
+        elif len(poles) == 1 and poles[0].n == 1:
+            p = poles[0].expr
+            d = var - p
+            r = (expr * d).cancel()
+            return Q, [r], [p], [1], delay, undef
+
+        # Find residues by solving system of equations.
+
+        syms = []
+        D = []
+        O = []
+        P = []
+        i = 1
+        denom_factored = One
+        for pole in poles:
+            for m in range(pole.n):
+                A = sym.Symbol('A_%d' % i)
+                p = pole.expr
+                d = (var - p) ** (m + 1)
+                syms.append(A)
+                D.append(d)
+                P.append(p)
+                O.append(m + 1)
+                i += 1
+            denom_factored *= (var - pole.expr) ** pole.n
+
+        rhs = Zero
+        for denom, residue in zip(D, syms):
+            # Could be more cunning to avoid using cancel
+            rhs += residue * (denom_factored / denom).cancel()
+
+        rhspoly = sym.poly(rhs, var)
+        lhspoly = sym.poly((expr * denom_factored).cancel(), var)
+
+        rc = rhspoly.all_coeffs()
+        lc = lhspoly.all_coeffs()
+        if len(lc) < len(rc):
+            lc = [0] * (len(rc) - len(lc)) + lc
+
+        A, _ = sym.linear_eq_to_matrix(rc, syms)
+
+        # Solve system of equations to find residues A_n.
+        R = list(matrix_inverse(A) * sym.Matrix(lc))
+
+        # Remove elements where the residue is zero.
+        R, P, O = self._prune_zero_residues(R, P, O)
+
+        return Q, R, P, O, delay, undef
+
+    def _combine_conjugates(self, R, D, P, O):
+
+        Rnew = []
+        Dnew = []
+
+        for m in range(len(R)):
+
+            r = R[m]
+            if r == 0:
+                continue
+            p = P[m]
+            o = O[m]
+            rc = r.conjugate()
+            pc = p.conjugate()
+
+            has_conjugate = False
+            for n in range(m + 1, len(R)):
+                if o != O[n] or pc != P[n] or rc != R[n]:
+                    continue
+                if o > 1:
+                    continue
+                R[n] = 0
+                has_conjugate = True
+                break
+
+            if has_conjugate:
+                Rnew.append(((self.var - pc) * r + (self.var - p) * rc).expand())
+                Dnew.append(((self.var - p) * (self.var - pc)).expand())
+            else:
+                Rnew.append(r)
+                Dnew.append(D[m])
+        return Rnew, Dnew
+
+
+    def as_QRD(self, combine_conjugates=False, damping=None):
+        """Decompose expression into Q, R, D, delay, undef where
+
+        expression = (Q + sum_n R_n / D_n) * exp(-delay * var) * undef"""
+
+        Q, R, P, O, delay, undef = self.as_QRPO(damping)
+
+        D = [(self.var - p)**o for p, o in zip(P, O)]
+
+        if combine_conjugates:
+            R, D = self._combine_conjugates(R, D, P, O)
+
         return Q, R, D, delay, undef
-            
