@@ -8,9 +8,11 @@ Copyright 2020--2021 Michael Hayes, UCECE
 
 from .dexpr import DiscreteExpression
 from .sequence import Sequence
-from .functions import Heaviside, DiracDelta, rect, sign
+from .functions import function_mapping
 from .extrafunctions import UnitStep, UnitImpulse, dtrect, dtsign
+from sympy import Heaviside
 from numpy import arange
+from warnings import warn
 
 
 class SequenceExpression(DiscreteExpression):
@@ -20,16 +22,24 @@ class SequenceExpression(DiscreteExpression):
 
         super(SequenceExpression, self).__init__(val, **assumptions)
 
-        mapping = {Heaviside: UnitStep,
-                   DiracDelta: UnitImpulse,
-                   rect: dtrect,
-                   sign: dtsign}
+        def remap_continuous_discrete(expr):
+        
+            def query(expr):
+                return expr.is_Function and expr.func in function_mapping
 
-        # Use discrete-time function variants, see also functions.py
-        for old, new in mapping.items():
-            if self.has(old):
-                self.expr = self.replace(old, new).expr                
-                
+            def value(expr):
+                # Note, Heaviside behaviour changed in SymPy-1.9
+                # The second arg now defaults to 1/2, so we need
+                # to remove it.
+                if expr.func == Heaviside:
+                    return UnitStep(expr.args[0])
+                return function_mapping[expr.func](*expr.args)
+
+            # Use discrete-time function variants, see also functions.py
+            return expr.replace(query, value)
+        
+        self.expr = remap_continuous_discrete(self.expr)
+        
     def first_index(self, ni=None):
 
         if ni is None:
@@ -88,10 +98,10 @@ class SequenceExpression(DiscreteExpression):
             # Should search to handle cases such as 1, 1, 0, 1 but
             # when to stop?
             if self(n1 - 1) != 0:
-                print('Warning: sequence truncated at n1=%d' % n1)
+                warn('Sequence truncated at n1=%d' % n1)
                 start_trunc = True
             if self(n2 + 1) != 0:
-                print('Warning: sequence truncated at n2=%d' % n2)
+                warn('Sequence truncated at n2=%d' % n2)
                 end_trunc = True
             
         elif isinstance(ni, tuple):
