@@ -4,7 +4,7 @@ Copyright 2014--2021 Michael Hayes, UCECE
 
 """
 
-# The following are Lcapy expressions.  For example, f is an
+# The following domain variables are Lcapy expressions.  For example, f is an
 # FourierDomainExpression object wrapping the fsym symbol.
 from .fexpr import f
 from .texpr import t
@@ -34,3 +34,5 @@ w0 = omega0
 
 W = Omega
 
+domain_vars = [t, f, s, omega, omega0, Omega, F, n, k, z, jw, jw0]
+domain_var_ids = [id(var) for var in domain_vars]
