@@ -288,7 +288,11 @@ class LcapyTester(unittest.TestCase):
         h = H.inverse_laplace()
         self.assertEqual2(h, exp(-t / 2) * Heaviside(t) / 2,
                           "Incorrect impulse response")        
-
+        H1 = a.transfer((1, 0), (2, 0))
+        self.assertEqual2(H1, H, "Incorrect transfer function")
+        H2 = a.transfer('R1', 'C1')
+        self.assertEqual2(H2, H, "Incorrect transfer function")
+        
 
     def test_VRC2(self):
         """Check VRC circuit with arbitrary s-domain source
@@ -736,3 +740,17 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.R1.is_independent_source, False, "R is_independent_source")                         
         self.assertEqual(a.R1.is_resistor, True, "E is_resistor")                
         
+
+    def test_TP(self):
+        """Test TP"""
+
+        n = Series(R(5)).chain(Shunt(R(10)))
+        b = n.Bmodel
+        c = b.circuit()
+        c.add('V 1 0 15; down')
+        self.assertEqual(c[3].V, 10, "Bmodel")                        
+        
+        d = n.Ymodel
+        e = d.circuit()
+        e.add('V 1 0 15; down')
+        self.assertEqual(e[3].V, 10, "Ymodel")                                
