@@ -30,8 +30,10 @@ class LcapyTester(unittest.TestCase):
                          "1 / s")
         self.assertEqual((s * 0 + 1).inverse_laplace(causal=True), DiracDelta(t),
                          "1")
+        self.assertEqual((s).inverse_laplace(causal=True), DiracDelta(t, 1),
+                         "s")        
         self.assertEqual((s * 0 + 10).inverse_laplace(causal=True), 10
-                         * DiracDelta(t), "0")
+                         * DiracDelta(t), "10")
         self.assertEqual(LaplaceDomainVoltage('V(s)').inverse_laplace(causal=True),
                          TimeDomainVoltage('v(t)'), "V(s)")
         self.assertEqual(LaplaceDomainVoltage('10 * V(s)').inverse_laplace(causal=True),
@@ -40,11 +42,28 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(TimeDomainVoltage('v(t)').laplace().inverse_laplace(causal=True),
                          TimeDomainVoltage('v(t)'), "v(t)")
         self.assertEqual(expr('1/(s+a)').inverse_laplace(causal=True), expr('exp(-a * t) * u(t)'), "1/(s+a)")
-        self.assertEqual(expr('1/(s**2)').inverse_laplace(causal=True), expr('t * u(t)'), "1/(s**2)")        
+        self.assertEqual(expr('1/(s**2)').inverse_laplace(causal=True), expr('t * u(t)'), "1/(s**2)")
+        self.assertEqual(expr('1/((s+3)**2)').inverse_laplace(causal=True), expr('t * u(t) * exp(-3 * t)'), "1/((s+3)**2)")        
+        self.assertEqual(expr('1/(s**3)').inverse_laplace(causal=True), expr('t**2 * u(t) / 2'), "1/(s**3)")                
         self.assertEqual(expr('s/(s+a)').inverse_laplace(causal=True), expr('-a * exp(-a * t) * u(t) + delta(t)'), "s/(s+a)")
         self.assertEqual(expr('s/(s**2+a**2)').inverse_laplace(causal=True), expr('cos(a * t) * u(t)'), "s/(s**2+a**2)")
-        self.assertEqual(expr('a/(s**2+a**2)').inverse_laplace(causal=True), expr('sin(a * t) * u(t)'), "a/(s**2+a**2)")                                                                           
+        self.assertEqual(expr('a/(s**2+a**2)').inverse_laplace(causal=True), expr('sin(a * t) * u(t)'), "a/(s**2+a**2)")
+        self.assertEqual(expr('(1/s - exp(-s)/s)/s').ILT(causal=True),
+                         expr('(t*u(t) + (1 - t)*u(t - 1))'), "(1/s - exp(-s)/s)/s")
+        self.assertEqual(expr('(1/s - exp(-s)/s)/s').expand().ILT(causal=True),
+                         expr('(t*u(t) + (1 - t)*u(t - 1))'), "(1/s - exp(-s)/s)/s")
 
+        self.assertEqual(expr('s*(1/s - exp(-5*s)/s)/(s + 1)').ILT(causal=True),
+                         expr('-exp(5 - t)*u(t - 5) + exp(-t)*u(t)'),
+                         "'s*(1/s - exp(-5*s)/s)/(s + 1)")
+
+        self.assertEqual(expr('s*(1/s - exp(-5*s)/s)/(s + 1)').expand().ILT(causal=True),
+                         expr('-exp(5 - t)*u(t - 5) + exp(-t)*u(t)'),
+                         "'s*(1/s - exp(-5*s)/s)/(s + 1)")        
+
+        A4 = (1/s - exp(-5*s)/s)/(s**2 + 707*s/500 + 1)
+        self.assertEqual(A4(t)(s), A4, str(A4))
+        
     def test_damped_sin(self):
 
         H1 = 2 / (2 * s ** 2 + 5 * s + 6)
@@ -86,5 +105,33 @@ class LcapyTester(unittest.TestCase):
 
         a = expr('Integral(3 * x(t - tau), (tau, 0, oo))')
         self.assertEqual(a(s), expr('3 * X(s) / s'), "3 * X / s")
+
+        a = expr('Integral(3 * x(tau), (tau, 0, t))')
+        self.assertEqual(a(s), expr('3 * X(s) / s'), "3 * X / s")        
         
- 
+        a = expr('x(t)').convolve(expr('g(t)'))
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")
+
+        a = expr('x(t)').convolve(expr('g(t)'), commutate=True)
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")        
+
+        a = expr('x(t)').convolve(expr('g(t)', causal=True))
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")
+
+        a = expr('x(t)').convolve(expr('g(t)', causal=True), commutate=True)
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")
+
+        a = expr('x(t)', causal=True).convolve(expr('g(t)'))
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")
+
+        a = expr('x(t)', causal=True).convolve(expr('g(t)', commutate=True))
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")
+
+        a = expr('x(t)', causal=True).convolve(expr('g(t)', causal=True))
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")
+
+        a = expr('x(t)', causal=True).convolve(expr('g(t)', causal=True,
+                                                    commutate=True))
+        self.assertEqual(a(s), expr('X(s) * G(s)'), "X(s) * G(s)")                                                                                               
+
+
